<div class="init-block" data-block-name="title" data-title="{{ title }}"></div>
<div class="init-block" data-block-name="menu" data-menu="{{ menu }}"></div>
<table style="margin: auto; font-size: 16pt;" cellspacing="2em">
  <tbody>
    {% for row in dashboard.items %}
      {{ row.render() }}
    {% endfor %}
  </tbody>
</table>
